//-------------------------------------------------------------------
//  Bryan C Warren
//  Modified by Axel Koch
//  to accommodate Game Pad
//  Independent Studies
//  Direct Input Interface Library (DIIL) Version 2.0
//-------------------------------------------------------------------
//
//  DirInput.cpp: Contains functions which can be called by other
//                WINAPI programs to make using Direct Input much
//                easier.
//
//  Functions Available:
//
//      CaptureFFdevice     - Initialization of DirectInput objects.
//                            Sets up interface variables to DirectX
//                            to allow other functions to work.
//                            Later iterations may include a non-
//                            force feedback version, also possibly
//                            keyboard, mouse, and driving wheel
//                            versions. Low Limit and High Limit set
//                            by user, default is -1000 to +1000.
//
//      GetDeviceType       - Returns the type of device that is currently
//                            connected. Return value is of type enum DeviceType
//                            and can be either JOYSTICK, GAMEPAD
//                            NO_DEVICE_FOUND or UNKNOWN_DEVICE.
//                            This function should only be called
//                            after 'CaptureFFdevice' has been called.
//
//      ReleaseFFdevice     - Allows Direct Input objects to be deleted and
//                            released when no longer needed.
//      
//      GetXYStatus         - Polls Device for joystick position.
//							  Corresponds to left joystick on game pad and the
//                            main axis on the joystick device. Parameter
//                            returned is of type POINT, containing X & Y values,
//                            in the range of Low Limit to High Limit.     
//
//		GetABStatus         - Polls device for secondary axis positions.
//                            Corresponds to right joystick on game pad and
//                            the rudder (A axis) and throttle (B axis) on the 
//							  joystick device. Parameter returned
//                            is of type POINT, containing A & B values,
//                            in the range of Low Limit to High Limit.
//
//      GetSliderStatus     - Only applicable to game pad! Always returns 0 for 
//							  joystick.
//                            Polls game pad for slider position.
//                            Parameter returned is of type LONG, in the range
//                            of Low Limit to High Limit.
//
//      ButtonPressed       - Polls device to see if button is pressed.
//                            True indicates button is pressed.
//                            Button Number passed in by user, from range
//                            of 1 to 32, default is 1.
//
//      GetPOVswitchStatus  - Polls device for point of view (POV) switch position.
//                            Parameter returned is of type enum POVstatus. 
//                            Values can be NORTH, NORTHEAST, EAST, SOUTHEAST,
//                            SOUTH, SOUTHWEST, WEST, and NORTHWEST. 
//                            NILL indicates center position.
//
//      SetupPeriodicForce  - Establishes parameters for a pulsating force according
//                            to a defined wave pattern (i.e. shaking effect).
//                            Creates an instance of a periodic effect which
//                            can then be started and stopped as necessary.
//                            Input parameters allowed are the period of the wave.
//                            Value defaults to 1/20 second.
//
//      AlterPeriodicForce  - Allows the periodic force effect to be changed
//                            without recreating the effect. Input parameters
//                            are magnitude, period, offset, and direction.
//                            For joystick, only input parameters magnitude and
//                            period will show a change in effect.
//                            Allowed value ranges are:
//                            magnitude: 0 to 10000
//                            period: 0.00 to 10.00 (game pad) 0.04 to 0.20 (joystick) 
//                            offset: -10000 to 10000 (game pad only)
//                            direction: 0 to 35999 (game pad only)
//
//      StartPeriodicForce  - Starts effect instances created by Setup
//                            Periodic Force.
//
//      StopPeriodicForce   - Stops effect instance created in Setup
//                            Periodic Force.
//                            
//      CreateConstantForce - Initializes and starts a constant force
//                            effect. Uses coordinate math to create
//                            direction and magnitude of force.
//                            Input parameters allowed are X Coordinate
//                            and Y Coordinate, as doubles, of relative origin
//                            and magnitude of force. Default values are 0, which
//                            creates a zero force with no direction. Expected
//                            range of values is -10000 to +10000. Out of range
//                            values will not cause error.
//                            Only applicable to joystick! A constant force
//                            cannot be created for the game pad device.
//
//      AlterConstantForce  - Allows the constant force effect to be changed
//                            without recreating the effect. Expected
//                            range of values is -10000 to +10000. Out of range
//                            values will not cause error.
//                            Only applicable to joystick!
//
//      StartConstantForce  - Starts effect instance created in Create
//                            Constant Force.
//                            Only applicable to joystick!
//
//      StopConstantForce   - Stops effect instance created in Create
//                            Constant Force.
//                            Only applicable to joystick!
//
//      StartForce          - Starts all effect instances created.
//                            Joystick: starts constant and periodic force effects.
//                            Game pad: same as Start Periodic Force.
//
//      StopForce           - Stops all effect instances created.
//                            Joystick: stops constant and periodic force effects.
//                            Game pad: same as Stop Periodic Force.
//
//  Assumptions:
//
//      1.  The first device found is the one you want to use.
//
//      2.  Only standard effects are desired. (User not interested in
//                                              customizing)
//
//      3.  Supported devices are force feedback joystick or game pad.
//
//      4.  Devices cannot be changed during run time. That is, 
//          user must restart program after device change.
//
//-------------------------------------------------------------------

#define   STRICT
#include <tchar.h>
#include <windows.h>
#include <windowsx.h>
#include <basetsd.h>
#include <mmsystem.h>
#include <dinput.h>
#include <math.h>


#define SAFE_DELETE(p)  { if(p) { delete (p);     (p)=NULL; } }
#define SAFE_RELEASE(p) { if(p) { (p)->Release(); (p)=NULL; } }

enum DeviceType{ NO_DEVICE_FOUND, JOYSTICK, GAMEPAD, UNKNOWN_DEVICE };
enum POVstatus { NILL, NORTH, NORTHEAST, EAST, SOUTHEAST, SOUTH, SOUTHWEST, WEST, NORTHWEST };


BOOL CALLBACK  DIEnumDevicesProc(   const DIDEVICEINSTANCE* lpddi, 
     							    VOID* pContext );
BOOL CALLBACK  EnumAxesCallback(    const DIDEVICEOBJECTINSTANCE* pdidoi,
                                    VOID* pContext );
DIJOYSTATE     PollDevice();

VOID           ReleaseFFdevice ();




static HINSTANCE             bry_hinst;
static HWND                  bry_hwnd               = NULL;

static LPDIRECTINPUT8        bry_DirectInput        = NULL;
static LPDIRECTINPUTDEVICE8  bry_DIDevice           = NULL; 
static LPDIRECTINPUTEFFECT   bry_ConstantEffect     = NULL;  // global effect object
static LPDIRECTINPUTEFFECT   bry_PeriodEffect       = NULL;  // global effect object

static DIPERIODIC            bry_diPeriodic         = {0};   // type-specific parameters
static DIENVELOPE            bry_diEnvelope         = {0};   // envelope
static DIEFFECT              bry_diEffectPeriodic   = {0};   // general parameters

static DICONSTANTFORCE       bry_diConstantForce    = {0};
static DIEFFECT              bry_diEffectConstant   = {0};   // general parameters

static DWORD                 bry_forceAxisCount     = 0;
static INT                   bry_lowRange           = 0;
static INT                   bry_highRange          = 0;
static INT                   bry_devType            = NO_DEVICE_FOUND;




//-------------------------------------------------------------------
//  Capture Force Feedback Device
//-------------------------------------------------------------------
BOOL  CaptureFFdevice( HWND hwnd,
                         INT lowLimit,
                         INT highLimit ) {

    HRESULT hr; 

    bry_hwnd        = hwnd;
    bry_lowRange          = lowLimit;
    bry_highRange         = highLimit;

    
	// Create Direct Input instance.
    hr = DirectInput8Create( GetModuleHandle(NULL),
                             DIRECTINPUT_VERSION,
                             IID_IDirectInput8,
                             (VOID**)&bry_DirectInput, NULL );

    if (FAILED(hr)) return hr; 


    // Capture ForceFeedback Device
    hr = bry_DirectInput->EnumDevices( DI8DEVCLASS_GAMECTRL, 
                                       DIEnumDevicesProc,
                                       NULL, 
                                       DIEDFL_ATTACHEDONLY | DIEDFL_FORCEFEEDBACK);


    if (FAILED(hr) || (bry_DIDevice == NULL) ) {
        ReleaseFFdevice();
		MessageBox( NULL, TEXT("No Force Feedback Device available."),  
                    TEXT("DirInput"), 
                    MB_ICONERROR | MB_OK );
        return FALSE;
    }


	DIDEVICEINSTANCE DiDeviceInstance;
	DiDeviceInstance.dwSize  = sizeof(DIDEVICEINSTANCE);

	if(!(DI_OK == bry_DIDevice->GetDeviceInfo(&DiDeviceInstance))){
		MessageBox(NULL,TEXT("Cannot get device info. Program exits!"),TEXT("DirInput"),MB_ICONERROR | MB_OK );
		return FALSE;
	}
	else{
		switch(DiDeviceInstance.dwDevType & 0xff){

		case DI8DEVTYPE_GAMEPAD:
			bry_devType = GAMEPAD; 
			break;
		case DI8DEVTYPE_JOYSTICK:
			bry_devType = JOYSTICK;
			break;
//      This API could possibly be extended in the future  
//      to also accommodate the two device classes below.
//		case DDI8DEVTYPE_DRIVING :
			//Device for steering.
//			break;
//		case DI8DEVTYPE_FLIGHT:
			//Controller for flight simulation. 
//			break;
		
		default:
			MessageBox(NULL,TEXT("Found unknown device!"),TEXT("DirInput"),MB_ICONERROR | MB_OK );
			bry_devType = UNKNOWN_DEVICE;
			return FALSE;
		}

	}
    
    // Set game data format
    hr = bry_DIDevice->SetDataFormat(&c_dfDIJoystick);

    if (FAILED(hr)) return hr;
 
    // Set cooperative level.
    hr = bry_DIDevice->SetCooperativeLevel(bry_hwnd,
                                           DISCL_EXCLUSIVE | DISCL_FOREGROUND);

    if (FAILED(hr)) return hr;
 

    // Turn off the joystick's autocenter feature.
    DIPROPDWORD DIPropAutoCenter;
 
    DIPropAutoCenter.diph.dwSize       = sizeof(DIPROPDWORD);
    DIPropAutoCenter.diph.dwHeaderSize = sizeof(DIPROPHEADER);
    DIPropAutoCenter.diph.dwObj        = 0;
    DIPropAutoCenter.diph.dwHow        = DIPH_DEVICE;
    DIPropAutoCenter.dwData            = FALSE;

    hr = bry_DIDevice->SetProperty( DIPROP_AUTOCENTER, 
                                    &DIPropAutoCenter.diph );

    if (FAILED(hr)) return hr;
 
    // Enumerate and count the axes of the joystick 
    hr = bry_DIDevice->EnumObjects( EnumAxesCallback,
                                   (VOID*)&bry_forceAxisCount,
                                    DIDFT_AXIS );

    if (FAILED(hr)) return hr;

    // This simple sample only supports one or two axis joysticks
	if( bry_forceAxisCount > 2 )
        bry_forceAxisCount = 2;
	
    return TRUE;
}   // end CaptureFFdevice




//-------------------------------------------------------------------
//  Get Device Type
//  - Get type of device that's currently connected
//    (GAMEPAD,JOYSTICK or UNKNOWN) 
//-------------------------------------------------------------------
DeviceType GetDeviceType() {
	if (!bry_DIDevice) return NO_DEVICE_FOUND;
	else if(bry_devType == GAMEPAD) return GAMEPAD;
	else if(bry_devType == JOYSTICK)return JOYSTICK;
	else return UNKNOWN_DEVICE;
}   //end GetDeviceType




//-------------------------------------------------------------------
//  Release Force Feedback Device
//  - End DirectInput Instance
//-------------------------------------------------------------------
void  ReleaseFFdevice () {

    // Unacquire the device one last time just in case 
    // the app tried to exit while the device is still acquired.
    if( bry_DIDevice ) 
        bry_DIDevice->Unacquire();
    
    // Release any DirectInput objects.
    SAFE_RELEASE( bry_ConstantEffect );
    SAFE_RELEASE( bry_PeriodEffect );
    SAFE_RELEASE( bry_DIDevice );
    SAFE_RELEASE( bry_DirectInput );
}   // end ReleaseFFdevice




//-------------------------------------------------------------------
//  Get XY Status
//  - Read device data for stick position.
//-------------------------------------------------------------------
POINT  GetXYStatus() {

    POINT posXY = {0, 0};

    DIJOYSTATE  js = PollDevice();           // DInput device state 

    posXY.x = js.lX;
    posXY.y = js.lY;

    return posXY;
}   // end GetXYStatus




//-------------------------------------------------------------------
//  Get AB Status
//  - Read device data for stick position.
//-------------------------------------------------------------------
POINT  GetABStatus() {

    POINT posAB = {0, 0};

    DIJOYSTATE  js = PollDevice();           // DInput device state 

    posAB.x = js.lRz;
    posAB.y = js.rglSlider[0];

    return posAB;
}   // end GetABStatus




//-------------------------------------------------------------------
//  Get Slider Status
//  - Read device data for slider position.
//-------------------------------------------------------------------
LONG  GetSliderStatus() {

    //Joystick does not have a slider control. Always return 0.
	if(bry_devType == JOYSTICK) return 0;
	
	LONG position   = 0;

    DIJOYSTATE js = PollDevice();           // DInput slider state 

    position        = js.lZ;

    return position;
}   // end GetSliderStatus




//-------------------------------------------------------------------
//  Button Pressed ?
//  - Read device data for trigger position.
//-------------------------------------------------------------------
BOOL  ButtonPressed( INT buttonNumber ) {

    BYTE button   = buttonNumber - 1;
    if( button > 31 ) button = 0;

    DIJOYSTATE js = PollDevice();           // DInput device state 

    button        = ( js.rgbButtons[button] & 0x80 );

    return button;
}   // end ButtonPressed




//-------------------------------------------------------------------
//  Get POVswitch Status
//  - Read device data for Point of View (POV) switch position.
//-------------------------------------------------------------------
POVstatus  GetPOVswitchStatus( ) {

    POVstatus POVswitch = NILL;

    DIJOYSTATE js = PollDevice();           // DInput device state 

    switch(js.rgdwPOV[0])
	{
	case 0:
		POVswitch = NORTH;
		break;
	case 4500:
		POVswitch = NORTHEAST;
		break;
	case 9000:
		POVswitch = EAST;
		break;
	case 13500:
		POVswitch = SOUTHEAST;
		break;
	case 18000:
		POVswitch = SOUTH;
		break;
	case 22500:
		POVswitch = SOUTHWEST;
		break;
	case 27000:
		POVswitch = WEST;
		break;
	case 31500:
		POVswitch = NORTHWEST;
		break;
	default:
		POVswitch = NILL;
	}

    return POVswitch;
}   // end GetPOVswitchStatus




//-------------------------------------------------------------------
//  Setup Periodic Force
//  - Initialization of pulsing effect.
//-------------------------------------------------------------------
HRESULT  SetupPeriodicForce( double period ) {

    HRESULT hr;
	//Periodic effect already exists
	if( NULL != bry_PeriodEffect ) return E_FAIL;

	  DWORD      dwAxes[4] = {DIJOFS_X, DIJOFS_Y, DIJOFS_RZ, DIJOFS_SLIDER(0)};
      LONG       lDirection[4] = {0, 0, 0, 0};

    // Initialize the type-specific parameters.
    bry_diPeriodic.dwMagnitude           = DI_FFNOMINALMAX; 
    bry_diPeriodic.lOffset               = 0; 
    bry_diPeriodic.dwPhase               = 0; 
    bry_diPeriodic.dwPeriod              = (DWORD)(period * DI_SECONDS); 
 
    bry_diEnvelope.dwSize                = sizeof(DIENVELOPE);
    bry_diEnvelope.dwAttackLevel         = 0; 
    bry_diEnvelope.dwAttackTime          = 0;//(DWORD)(attackTime * DI_SECONDS); 
    bry_diEnvelope.dwFadeLevel           = 0; 
    bry_diEnvelope.dwFadeTime            = 0;//(DWORD)(1.0 * DI_SECONDS); 
 
    // set up the basic effect parameters. 
    bry_diEffectPeriodic.dwSize                  = sizeof(DIEFFECT); 
    bry_diEffectPeriodic.dwFlags                 = DIEFF_POLAR | DIEFF_OBJECTOFFSETS; 
    bry_diEffectPeriodic.dwDuration              = INFINITE;
 
    bry_diEffectPeriodic.dwSamplePeriod          = 0;                 // = default 
    bry_diEffectPeriodic.dwGain                  = DI_FFNOMINALMAX;   // no scaling
    bry_diEffectPeriodic.dwTriggerButton         = DIEB_NOTRIGGER;
    bry_diEffectPeriodic.dwTriggerRepeatInterval = 0;      
    bry_diEffectPeriodic.cAxes                   = bry_forceAxisCount; 
    bry_diEffectPeriodic.rgdwAxes                = dwAxes; 
    bry_diEffectPeriodic.rglDirection            = &lDirection[0]; 
    bry_diEffectPeriodic.lpEnvelope              = 0; 
    bry_diEffectPeriodic.cbTypeSpecificParams    = sizeof(bry_diPeriodic);
    bry_diEffectPeriodic.lpvTypeSpecificParams   = &bry_diPeriodic;

    hr = bry_DIDevice->CreateEffect( GUID_Square,               // GUID from enumeration
                                     &bry_diEffectPeriodic,     // where the data is
                                     &bry_PeriodEffect,         // where to put interface pointer
                                     NULL );                    // no aggregation

    if (FAILED(hr)) return hr;

    return S_OK;
}   // end SetupPeriodicForce




//-------------------------------------------------------------------
//  Alter Periodic Force
//  - Change pulsing effect.
//-------------------------------------------------------------------
HRESULT		   AlterPeriodicForce( int magnitude,
								   double period,
								   int offset,
								   int direction ) {
	if(bry_devType == JOYSTICK){
		//If the connected device is a force feedback joystick
		//then restrict the input to values that make sense and 
		//won't break the device
		offset = 0;
		direction = 0;
		if(period < 0.04) period = 0.04;
		if(period > 0.20) period = 0.20;
	}

	HRESULT hr;
	
	if(magnitude < 0) magnitude = 0;
	if(offset < -(DI_FFNOMINALMAX/2)) offset = -(DI_FFNOMINALMAX/2);
	if(offset > (DI_FFNOMINALMAX/2)) offset = DI_FFNOMINALMAX/2;
	if(abs(offset + magnitude) > DI_FFNOMINALMAX)
	    magnitude = DI_FFNOMINALMAX - abs(offset);
	if(direction < 0) direction = 0;
	if(direction > 35999) direction = 35999;

	LONG       lDirection[4] = {0, 0, 0, 0};
	lDirection[0] = direction;

	bry_diPeriodic.dwMagnitude           = (DWORD)magnitude; 
    bry_diPeriodic.lOffset               = (DWORD)offset; 
    bry_diPeriodic.dwPeriod              = (DWORD)(period * DI_SECONDS);
	bry_diEnvelope.dwSize                = sizeof(DIENVELOPE);

	hr = bry_PeriodEffect->SetParameters(&bry_diEffectPeriodic, DIEP_TYPESPECIFICPARAMS);
	if (FAILED(hr)) return hr;
	hr = bry_PeriodEffect->SetParameters(&bry_diEffectPeriodic, DIEP_ENVELOPE);
	if (FAILED(hr)) return hr;
	bry_diEffectPeriodic.rglDirection = &lDirection[0];
	hr = bry_PeriodEffect->SetParameters(&bry_diEffectPeriodic, DIEP_DIRECTION);
	if (FAILED(hr)) return hr;

    return S_OK;
}   // end AlterPeriodicForce




//-------------------------------------------------------------------
//  Start Periodic Force / Stop Periodic Force
//  - Start & Stop periodic effect.
//-------------------------------------------------------------------
void  StartPeriodicForce () {
    
    bry_DIDevice->Acquire();

    if (bry_PeriodEffect)
        bry_PeriodEffect->Start(1, 0);
}   // end StartPeriodicForce

void  StopPeriodicForce () {

    bry_DIDevice->Acquire();

    if (bry_PeriodEffect)
        bry_PeriodEffect->Stop();
}   // end StopPeriodicForce




//-------------------------------------------------------------------
//  Create Constant Force
//  - Creates and starts initial Constant Force.
//-------------------------------------------------------------------
HRESULT  CreateConstantForce( double bry_XForce,
                              double bry_YForce ) {
	
	//Cannot create a constant force for a game pad
	if(bry_devType == GAMEPAD) return E_FAIL;
	//Constant effect already created
	if( NULL != bry_ConstantEffect ) return E_FAIL;

    HRESULT hr;

    DWORD rgdwAxes[2]     = { DIJOFS_X, DIJOFS_Y };

    LONG rglDirection[2] = { 0, 0 };
    rglDirection[0] = (LONG)bry_XForce;
    rglDirection[1] = (LONG)bry_YForce;
    
    bry_diConstantForce.lMagnitude = (DWORD)sqrt( bry_XForce * bry_XForce +
                                         bry_YForce * bry_YForce );

    ZeroMemory( &bry_diEffectConstant, sizeof(bry_diEffectConstant) );
    bry_diEffectConstant.dwSize                  = sizeof(DIEFFECT);
    bry_diEffectConstant.dwFlags                 = DIEFF_CARTESIAN | DIEFF_OBJECTOFFSETS;
    bry_diEffectConstant.dwDuration              = INFINITE;
    bry_diEffectConstant.dwSamplePeriod          = 0;
    bry_diEffectConstant.dwGain                  = DI_FFNOMINALMAX;
    bry_diEffectConstant.dwTriggerButton         = DIEB_NOTRIGGER;
    bry_diEffectConstant.dwTriggerRepeatInterval = 0;
    bry_diEffectConstant.cAxes                   = bry_forceAxisCount;
    bry_diEffectConstant.rgdwAxes                = rgdwAxes;
    bry_diEffectConstant.rglDirection            = rglDirection;
    bry_diEffectConstant.lpEnvelope              = 0;
    bry_diEffectConstant.cbTypeSpecificParams    = sizeof(DICONSTANTFORCE);
    bry_diEffectConstant.lpvTypeSpecificParams   = &bry_diConstantForce;
    bry_diEffectConstant.dwStartDelay            = 0;

    // Create the prepared effect
    hr = bry_DIDevice->CreateEffect( GUID_ConstantForce, 
                                     &bry_diEffectConstant,
                                     &bry_ConstantEffect,
                                     NULL );
    if( FAILED(hr)) return hr;

    if( NULL == bry_ConstantEffect )
        return E_FAIL;

    return S_OK;
}

 


//-------------------------------------------------------------------
//  Alter Constant Force
//  - Updates direction & magnitude of Constant Force.
//-------------------------------------------------------------------
HRESULT  AlterConstantForce( double bry_XForce,
                             double bry_YForce ) {

	if(bry_devType == GAMEPAD) return E_FAIL;  //Game pad cannot have a constant force

    // Modifying an effect is basically the same as creating a new one, except
    // you need only specify the parameters you are modifying
    LONG rglDirection[2]                     = { (LONG)bry_XForce, (LONG)bry_YForce };

    bry_diConstantForce.lMagnitude               = (DWORD)sqrt(bry_XForce * bry_XForce +
                                                           bry_YForce * bry_YForce );

    bry_diEffectConstant.rglDirection            = rglDirection;
    bry_diEffectConstant.lpvTypeSpecificParams   = &bry_diConstantForce;
    bry_diEffectConstant.lpEnvelope              = 0;

    // Now set the new parameters and start the effect immediately.
    return bry_ConstantEffect->SetParameters( &bry_diEffectConstant,
                                              DIEP_DIRECTION |
                                              DIEP_TYPESPECIFICPARAMS | DIEP_START );
}




//-------------------------------------------------------------------
//  Start Constant Force / Stop Constant Force
//  - Start and stop constant effect.
//-------------------------------------------------------------------
void  StartConstantForce () {
    
    bry_DIDevice->Acquire();

    if (bry_ConstantEffect)
        bry_ConstantEffect->Start(1, 0);

}   // end StartConstantForce

void  StopConstantForce () {

    bry_DIDevice->Acquire();

    if (bry_ConstantEffect)
        bry_ConstantEffect->Stop();

}   // end StopConstantForce




//-------------------------------------------------------------------
//  Start Force / Stop Force
//  - Start & Stop both effects.
//-------------------------------------------------------------------
void  StartForce () {
    
    bry_DIDevice->Acquire();

	if (bry_devType == JOYSTICK && bry_ConstantEffect)
        bry_ConstantEffect->Start(1, 0);

    if (bry_PeriodEffect)
        bry_PeriodEffect->Start(1, 0);
}   // end StartForce

void  StopForce () {

    bry_DIDevice->Acquire();

    if (bry_devType == JOYSTICK && bry_ConstantEffect)
        bry_ConstantEffect->Stop();

    if (bry_PeriodEffect)
        bry_PeriodEffect->Stop();
}   // end StopForce






//***********  Helper functions - not exposed by the DirInput API ***********




//-------------------------------------------------------------------
//  Direct Input Enumerate Devices Process Callback
//-------------------------------------------------------------------
BOOL CALLBACK DIEnumDevicesProc( const DIDEVICEINSTANCE* lpddi, 
                                 LPVOID pContext ) {

    LPDIRECTINPUTDEVICE8 pDevice;
    HRESULT              hr;
    
    // Obtain an interface to the enumerated force feedback device.
    hr = bry_DirectInput->CreateDevice( lpddi->guidInstance, &pDevice, NULL );
    
    // If it failed, then we can't use this device for some
    // bizarre reason.  (Maybe the user unplugged it while we
    // were in the middle of enumerating it.)  So continue enumerating
    if( FAILED(hr) ) 
        return DIENUM_CONTINUE;
    
    // We successfully created an IDirectInputDevice8.  So stop looking 
    // for another one.
    bry_DIDevice = pDevice;

    return DIENUM_STOP;
}   // end DIEnumDevicesProc




//-------------------------------------------------------------------
//  Enumerate Axes Callback
//-------------------------------------------------------------------
BOOL CALLBACK EnumAxesCallback( const DIDEVICEOBJECTINSTANCE* pdidoi,
                                VOID* pContext ) {

    DWORD* pdwNumForceFeedbackAxis = (DWORD*) pContext;

    if( (pdidoi->dwFlags & DIDOI_FFACTUATOR) != 0 )
        (*pdwNumForceFeedbackAxis)++;

    HWND hDlg = (HWND)pContext;

    if (pdidoi->dwType & DIDFT_AXIS)
    {
        DIPROPRANGE DIPropRange;
    
        DIPropRange.diph.dwSize       = sizeof(DIPROPRANGE);
        DIPropRange.diph.dwHeaderSize = sizeof(DIPROPHEADER);
        DIPropRange.diph.dwHow        = DIPH_BYID;
        DIPropRange.diph.dwObj        = pdidoi->dwType;
        DIPropRange.lMin              = bry_lowRange;
        DIPropRange.lMax              = bry_highRange;

        if( FAILED( bry_DIDevice->SetProperty( DIPROP_RANGE, &DIPropRange.diph)))
            return DIENUM_STOP;
    }

    return DIENUM_CONTINUE;
}




//-------------------------------------------------------------------
//  Poll Device
//  - Access Direct Input object for device data
//-------------------------------------------------------------------
DIJOYSTATE PollDevice() {

    HRESULT hr;

    DIJOYSTATE js;           // DInput device state 
	hr = bry_DIDevice->Acquire();

	hr = bry_DIDevice->Poll();

    if( FAILED(hr) ) {
  
        // DInput is telling us that the input stream has been
        // interrupted. We aren't tracking any state between polls, so
        // we don't have any special reset that needs to be done. We
        // just re-acquire and try again.
        hr = bry_DIDevice->Acquire();

        while(hr == DIERR_INPUTLOST)
            hr = bry_DIDevice->Acquire();
    }

    hr = bry_DIDevice->GetDeviceState( sizeof(DIJOYSTATE), &js );

    return js;
}   // end PollDevice





 



 









 
