//-------------------------------------------------------------------
//  Bryan C Warren
//  Modified by Axel Koch
//  to accommodate Game Pad
//  Independent Studies
//  Direct Input Interface Library (DIIL) Version 2.0
//-------------------------------------------------------------------
//
//  DirInput.cpp: Contains functions which can be called by other
//                WINAPI programs to make using Direct Input much
//                easier.
//
//  Functions Available:
//
//      CaptureFFdevice     - Initialization of DirectInput objects.
//                            Sets up interface variables to DirectX
//                            to allow other functions to work.
//                            Later iterations may include a non-
//                            force feedback version, also possibly
//                            keyboard, mouse, and driving wheel
//                            versions. Low Limit and High Limit set
//                            by user, default is -1000 to +1000.
//
//      GetDeviceType       - Returns the type of device that is currently
//                            connected. Return value is of type enum DeviceType
//                            and can be either JOYSTICK, GAMEPAD
//                            NO_DEVICE_FOUND or UNKNOWN_DEVICE.
//                            This function should only be called
//                            after 'CaptureFFdevice' has been called.
//
//      ReleaseFFdevice     - Allows Direct Input objects to be deleted and
//                            released when no longer needed.
//      
//      GetXYStatus         - Polls Device for joystick position.
//							  Corresponds to left joystick on game pad and the
//                            main axis on the joystick device. Parameter
//                            returned is of type POINT, containing X & Y values,
//                            in the range of Low Limit to High Limit.     
//
//		GetABStatus         - Polls device for secondary axis positions.
//                            Corresponds to right joystick on game pad and
//                            the rudder (A axis) and throttle (B axis) on the 
//							  joystick device. Parameter returned
//                            is of type POINT, containing A & B values,
//                            in the range of Low Limit to High Limit.
//
//      GetSliderStatus     - Only applicable to game pad! Always returns 0 for 
//							  joystick.
//                            Polls game pad for slider position.
//                            Parameter returned is of type LONG, in the range
//                            of Low Limit to High Limit.
//
//      ButtonPressed       - Polls device to see if button is pressed.
//                            True indicates button is pressed.
//                            Button Number passed in by user, from range
//                            of 1 to 32, default is 1.
//
//      GetPOVswitchStatus  - Polls device for point of view (POV) switch position.
//                            Parameter returned is of type enum POVstatus. 
//                            Values can be NORTH, NORTHEAST, EAST, SOUTHEAST,
//                            SOUTH, SOUTHWEST, WEST, and NORTHWEST. 
//                            NILL indicates center position.
//
//      SetupPeriodicForce  - Establishes parameters for a pulsating force according
//                            to a defined wave pattern (i.e. shaking effect).
//                            Creates an instance of a periodic effect which
//                            can then be started and stopped as necessary.
//                            Input parameters allowed are the period of the wave.
//                            Value defaults to 1/20 second.
//
//      AlterPeriodicForce  - Allows the periodic force effect to be changed
//                            without recreating the effect. Input parameters
//                            are magnitude, period, offset, and direction.
//                            For joystick, only input parameters magnitude and
//                            period will show a change in effect.
//                            Allowed value ranges are:
//                            magnitude: 0 to 10000
//                            period: 0.00 to 10.00 (game pad) 0.04 to 0.20 (joystick) 
//                            offset: -10000 to 10000 (game pad only)
//                            direction: 0 to 35999 (game pad only)
//
//      StartPeriodicForce  - Starts effect instances created by Setup
//                            Periodic Force.
//
//      StopPeriodicForce   - Stops effect instance created in Setup
//                            Periodic Force.
//                            
//      CreateConstantForce - Initializes and starts a constant force
//                            effect. Uses coordinate math to create
//                            direction and magnitude of force.
//                            Input parameters allowed are X Coordinate
//                            and Y Coordinate, as doubles, of relative origin
//                            and magnitude of force. Default values are 0, which
//                            creates a zero force with no direction. Expected
//                            range of values is -10000 to +10000. Out of range
//                            values will not cause error.
//                            Only applicable to joystick! A constant force
//                            cannot be created for the game pad device.
//
//      AlterConstantForce  - Allows the constant force effect to be changed
//                            without recreating the effect. Expected
//                            range of values is -10000 to +10000. Out of range
//                            values will not cause error.
//                            Only applicable to joystick!
//
//      StartConstantForce  - Starts effect instance created in Create
//                            Constant Force.
//                            Only applicable to joystick!
//
//      StopConstantForce   - Stops effect instance created in Create
//                            Constant Force.
//                            Only applicable to joystick!
//
//      StartForce          - Starts all effect instances created.
//                            Joystick: starts constant and periodic force effects.
//                            Game pad: same as Start Periodic Force.
//
//      StopForce           - Stops all effect instances created.
//                            Joystick: stops constant and periodic force effects.
//                            Game pad: same as Stop Periodic Force.
//
//  Assumptions:
//
//      1.  The first device found is the one you want to use.
//
//      2.  Only standard effects are desired. (User not interested in
//                                              customizing)
//
//      3.  Supported devices are force feedback joystick or game pad.
//
//      4.  Devices cannot be changed during run time. That is, 
//          user must restart program after device change.
//
//-------------------------------------------------------------------

#ifndef _DIRINPUT_H
#define _DIRINPUT_H

extern enum DeviceType{ NO_DEVICE_FOUND, JOYSTICK, GAMEPAD, UNKNOWN_DEVICE };
extern enum POVstatus { NILL, NORTH, NORTHEAST, EAST, SOUTHEAST, SOUTH, SOUTHWEST, WEST, NORTHWEST }; 


extern BOOL        CaptureFFdevice(   HWND hwnd,
                                      INT lowLimit  = -1000,
                                      INT highLimit =  1000 );
extern DeviceType  GetDeviceType();

extern void        ReleaseFFdevice();

extern POINT       GetXYStatus();

extern POINT       GetABStatus();

extern LONG		   GetSliderStatus();

extern BOOL        ButtonPressed(  INT buttonNumber = 1  );

extern POVstatus   GetPOVswitchStatus();

extern HRESULT     SetupPeriodicForce(  double period = 0.05 );

extern HRESULT	   AlterPeriodicForce(  int magnitude = 10000,
									    double period = 0.05,
									    int offset    = 0,
										int direction = 0    );

extern void        StartPeriodicForce ();

extern void        StopPeriodicForce  ();

extern HRESULT     CreateConstantForce( double g_nXForce = 0,
									    double g_nYForce = 0 );

extern HRESULT     AlterConstantForce(  double g_nXForce,
                                        double g_nYForce );

extern void        StartConstantForce ();

extern void        StopConstantForce  ();


extern void        StartForce ();

extern void        StopForce ();



#endif  //_DIRINPUT_H